import Circle from "class/Circle";
export default class {
    constructor(x, y, size= 20, dir = 0, acceleration = 0.0005,
        turnSpeed = 0.005, friction = 0.0001, speed = 0, maxSpeed = 0.5) {
        this.x = x;
        this.y = y;
        this.size = size;
        this.dir = dir;
        this.acceleration = acceleration;
        this.friction = friction;
        this.turnSpeed = turnSpeed;
        this.speed = speed;
        this.maxSpeed = maxSpeed;
    }

    draw(ctx) {
        ctx.font = `${this.size}px serif`;
        ctx.textAlign = "center";
        ctx.textBaseline = "middle";
        ctx.translate(this.x, this.y);
        ctx.rotate(this.dir + Math.PI / 4);
        ctx.fillText('🚀', 0, 0);
        ctx.setTransform(1, 0, 0, 1, 0, 0);    
    }

    turn(dt, clockwise = true) {
        let turnRad = this.turnSpeed * dt;
        if (clockwise) {
            this.dir += turnRad;
        } else {
            this.dir -= turnRad;
        }
    }

    move(dt, ctx = null) {
        let distX = this.speed * dt * Math.cos(this.dir);
        let distY = this.speed * dt * Math.sin(this.dir);
        this.x += distX;
        this.y += distY;
        // Stuck in the canvas (if given)
        if (ctx == null) return;
        let gap = this.size / 2;
        if (this.x - gap < 0) {
            this.x = gap;
        }
        if (this.y - gap < 0) {
            this.y = gap;
        }
        let width = ctx.canvas.width;
        let height = ctx.canvas.height;
        if (this.x + gap > width) {
            this.x = width - gap;
        }
        if (this.y + gap > height) {
            this.y = height - gap;
        }
    }

    thruster(dt, reverse = false) {
        let speed = this.acceleration * dt;
        if (reverse) {
            this.speed -= speed;
        } else {
            this.speed += speed;
        }
        this.speed = Math.max(0, Math.min(this.speed, this.maxSpeed));
    }

    getColisionCircle() {
        return {x: this.x, y: this.y, r: this.size/2};
    }

    fire() {
        let c = new Circle(this.x, this.y, 5, 0.5);        
        return c;
    }

}