export default class {

  constructor(x, y, size, dir = 0, acceleration = 0.0005, turnSpeed = 0.005, friction = 0.0001, speed = 0, maxSpeed = 0.5) {
    this.x = x;
    this.y = y;
    this.size = size;
    this.dir = dir;
    this.acceleration = acceleration;
    this.friction = friction;
    this.turnSpeed = turnSpeed;
    this.speed = speed;
    this.maxSpeed = maxSpeed;
  }

  draw(ctx) {
    ctx.font = `${this.size}px serif`;
    ctx.textAlign = "center";
    ctx.textBaseline = "middle";
    ctx.translate(this.x, this.y);
    // correction by Pi/4: the initial angle of the unicode rocket
    ctx.rotate(this.dir + Math.PI / 4);
    ctx.fillText("🚀", 0, 0);
    ctx.setTransform(1, 0, 0, 1, 0, 0);
  }

  move(deltaT, ctx = null) {
    var distX = this.speed * deltaT * Math.cos(this.dir);
    var distY = this.speed * deltaT * Math.sin(this.dir);
    this.x += distX;
    this.y += distY;
    // Stuck in the canvas (if given)
    if (ctx == null) return;
    let gap = this.size / 2;
    if (this.x - gap < 0) {
      this.x = gap;
    }
    if (this.y - gap < 0) {
      this.y = gap;
    }
    let width = ctx.canvas.width;
    let height = ctx.canvas.height;
    if (this.x + gap > width) {
      this.x = width - gap;
    }
    if (this.y + gap > height) {
      this.y = height - gap;
    }
  }

  turn(deltaT, clockwise = true) {
    let deltaDir = this.turnSpeed * deltaT;
    this.dir += clockwise ? deltaDir : -deltaDir;
  }

  thrust(deltaT, reverse = false) {
    let deltaSpeed = this.acceleration * deltaT;
    let newSpeed = this.speed + (reverse ? -deltaSpeed :  deltaSpeed);
    newSpeed = Math.max(0, newSpeed);
    newSpeed = Math.min(this.maxSpeed, newSpeed);
    this.speed = newSpeed;
  }

  applyFriction(deltaT) {
    let deltaSpeed = this.friction * deltaT;
    let newSpeed = this.speed - deltaSpeed;
    newSpeed = Math.max(0, newSpeed);
    this.speed = newSpeed;
  }

}