import Keyboard from "class/Keyboard";
import Automaton from "class/LifeLikeAutomaton";
import AutomatonInFlatTorus from "class/LifeLikeAutomaton/InFlatTorus";
import randomColor from "randomcolor";

const KEYBOARD = new Keyboard();

let animation = {
  targetFps: 4,
  maxFps: 250,
  fps: 4,
  nbIter: 0,
  aliveProb: 0.10,
  tileSize: 14,
  tileSizeMin: 4,
  tileSizeMax: 40,
  birthRule: [false, false, false, true, false, false, false, false, false],
  survivalRule: [false, false, true, true, false, false, false, false, false],
  mapFlatTorus: true,
  automaton: null,
  timer: null,
  lastTime: null,
  ctx: $("canvas").get(0).getContext("2d"),

  generateAutomaton() {
    this.stop();
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    let x = Math.floor(this.ctx.canvas.width / this.tileSize);
    let y = Math.floor(this.ctx.canvas.height / this.tileSize);
    if (this.mapFlatTorus) {
      this.automaton = new AutomatonInFlatTorus(x, y, this.tileSize, randomColor());
    } else {
      this.automaton = new Automaton(x, y, this.tileSize, randomColor());
    }
    this.automaton.randomize(this.aliveProb);
    this.nbIter = 0;
    $("#nbIter").text(this.nbIter);
    this.automaton.draw(this.ctx);
    this.start();
  },

  updateAndDraw(deltaT) {
    this.automaton.applyRule(this.birthRule, this.survivalRule, this.ctx);
    this.fps = this.fps * 0.9 + 1000 / deltaT * 0.1;
    if (this.nbIter % this.targetFps == 0) {
      $("#fps").text(Math.round(this.fps));
    }
    this.nbIter++;
    $("#nbIter").text(this.nbIter);
  },

  start() {
    this.lastTime = window.performance.now();
    this.fps = this.targetFps;
    this.timer = setInterval(() => {
      let now = window.performance.now();
      let deltaT = now - this.lastTime;
      this.lastTime = now;
      this.updateAndDraw(deltaT);
    }, 1000 / this.targetFps);
  },

  stop() {
    clearInterval(this.timer);
    this.timer = null;
  },

  togglePause() {
    if (this.timer) {
      this.stop();
    } else {
      this.start()
    }
  },

  setTileSize(tileSize) {
    if (tileSize < this.tileSizeMin || tileSize > this.tileSizeMax) return;
    this.tileSize = tileSize;
    $("#tileSize").text(this.tileSize);
  },

  setTargetFps(fps) {
    if (fps < 1 || fps > this.maxFps) return;
    this.targetFps = fps;
    $("#fps-traget").text(this.targetFps);
    if (this.timer) {
      this.stop();
      this.start();
    }
  },

  setAliveProb(aliveProb) {
    if (aliveProb < 0) aliveProb = 0;
    if (aliveProb > 1) aliveProb = 1;
    this.aliveProb = aliveProb;
    let prctAliveProb = Math.round(this.aliveProb * 100);
    $("#aliveProb").text(`${prctAliveProb}%`);
  },

  updateDomRule(type, num) {
    let domRule = $(`.rule[data-rule-type="${type}"][data-rule-num="${num}"]`);
    let rule = type == 's' ? this.survivalRule : this.birthRule;
    if (rule[num]) {
      domRule.addClass('apply');
    } else {
      domRule.removeClass('apply');
    }
  },

  toggleRule(type, num) {
    if (type == "s") {
      this.survivalRule[num] = !this.survivalRule[num];
    } else {
      this.birthRule[num] = !this.birthRule[num];
    }
    this.updateDomRule(type, num);
  },

  updateDomMapType() {
    $("#map").text(this.mapFlatTorus ? $("#map").data('flat-torus') : $("#map").data('rectangle'));
  },

  toggleMapType() {
    this.mapFlatTorus = !this.mapFlatTorus;
    this.updateDomMapType();
    this.generateAutomaton();
  },

  init() {
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    KEYBOARD.onKey('p', () => this.togglePause());
    KEYBOARD.onKey('w', () => this.setTargetFps(this.targetFps + 1));
    KEYBOARD.onKey('s', () => this.setTargetFps(this.targetFps - 1));
    KEYBOARD.onKey('d', () => this.setAliveProb(this.aliveProb + 0.01));
    KEYBOARD.onKey('a', () => this.setAliveProb(this.aliveProb - 0.01));
    KEYBOARD.onKey('z', () => this.setTileSize(this.tileSize + 1));
    KEYBOARD.onKey('h', () => this.setTileSize(this.tileSize - 1));
    KEYBOARD.onKey('r', () => this.generateAutomaton());
    KEYBOARD.onKey('m', () => this.toggleMapType());
    $(".rule").on("click", event => {
      let rule = $(event.currentTarget);
      let type = rule.data("rule-type");
      let num = rule.data("rule-num");
      this.toggleRule(type, num);
    });
    $(this.ctx.canvas).on("click", event => {
      let rect = this.ctx.canvas.getBoundingClientRect();
      let x = Math.floor((event.clientX - rect.left) / this.tileSize);
      let y = Math.floor((event.clientY - rect.top) / this.tileSize);
      this.automaton.toggleState(x, y, this.ctx);
    });
    this.setTargetFps(this.targetFps);
    this.setAliveProb(this.aliveProb);
    this.setTileSize(this.tileSize);
    this.updateDomMapType();
    this.survivalRule.forEach((val, num) => this.updateDomRule('s', num));
    this.birthRule.forEach((val, num) => this.updateDomRule('b', num));
    this.generateAutomaton();
  }

}

animation.init();