import mainloop from "MainLoop.js";
import Spaceship from "class/Spaceship";
import Keyboard from "class/Keyboard";
import Circle from "class/Circle";
import {getRandomInt} from "lib/Math";

const KEYBOARD = new Keyboard();
const targets = new Map();
let score;
let elapsedTime = 0;
let bullet = null;

let spaceship = new Spaceship(100,100);
let ctx = $("canvas").get(0).getContext("2d");
ctx.canvas.width = ctx.canvas.clientWidth;
ctx.canvas.height = ctx.canvas.clientHeight;

let generateTarget = () => {
    let x = getRandomInt(20, ctx.canvas.width - 20);
    let y = getRandomInt(20, ctx.canvas.height - 20);
    let c = new Circle(x, y, 20);
    while (c.isCollidingWith(spaceship.getColisionCircle())) {
        x = getRandomInt(20, ctx.canvas.width - 20);
        y = getRandomInt(20, ctx.canvas.height - 20);
        c = new Circle(x, y, 20);
    }
    targets.set(c, c);
}
let spawnTimer = setInterval(generateTarget, 1000);

let update = (dt) => {
    elapsedTime += dt;
    if (KEYBOARD.isKeyDown('a')) {
        spaceship.turn(dt, false);
    }
    if (KEYBOARD.isKeyDown('d')) {
        spaceship.turn(dt, true);
    }
    if (KEYBOARD.isKeyDown('w')) {
        spaceship.thruster(dt);
    }
    if (KEYBOARD.isKeyDown('s')) {
        spaceship.thruster(dt, true);
    }
    if (KEYBOARD.isKeyDown(' ')) {
        bullet = spaceship.fire();
    }
    if (bullet) {
        bullet.move(spaceship.dir, dt);
    }
    spaceship.move(dt, ctx);
    targets.forEach(target => {
        if (target.isCollidingWith(spaceship.getColisionCircle())) {
            targets.delete(target);
        }
    });        
    if (targets.size >= 10) {
        mainloop.stop();
        clearInterval(spawnTimer);
    }
    score = Math.round(elapsedTime / 100);
};
let draw = (dt) => {
    ctx.canvas.width = ctx.canvas.clientWidth;
    ctx.canvas.height = ctx.canvas.clientHeight;
    spaceship.draw(ctx);
    targets.forEach(target => target.draw(ctx));  
    if (bullet) {
        bullet.draw(ctx);
        console.log("draw")
    }
    ctx.font = "20px serif";
    ctx.fillStyle = "white";
    ctx.fillText(`Score: ${score}` , 60, 20);  
};

mainloop
    .setUpdate(update)
    .setDraw(draw)
    .start();