import Keyboard from "class/Keyboard";
import BouncingCircle from "class/Circle/Bouncing";
import {getRandomInt} from "lib/Math";

const KEYBOARD = new Keyboard();

let gravity = {
  ctx: $("canvas").get(0).getContext("2d"),
  lastTick: null,
  elapsedTime: 0,
  circles: [],
  buffer: document.createElement("canvas").getContext("2d"),

  tick(now) {
    requestAnimationFrame(now => this.tick(now));
    let deltaT = now - this.lastTick;
    this.lastTick = now;
    //if less than 30fps (Δt > ~33,3 ms) drop the frame
    if (deltaT > 34) return;
    this.elapsedTime += deltaT;
    this.circles.forEach(c => {
      c.move(deltaT);
      c.applyFriction(deltaT);
      c.applyFloorFriction(deltaT);
      c.applyGravity(deltaT);
      c.bounceOffTheWalls(true);
    });
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    this.circles.forEach(c => c.draw(this.ctx));
  },

  generateCircle() {
    let radius = getRandomInt(5,50);
    let velocityX = getRandomInt(-10, 10) / 10;
    let velocityY = getRandomInt(-20, 0) / 10;
    let c =  new BouncingCircle(this.ctx.canvas.width / 2, this.ctx.canvas.height / 4, radius, velocityX, velocityY, this.ctx.canvas.width, this.ctx.canvas.height);
    this.circles.push(c);
    // delete old ones
    if (this.circles.length > 100) {
      this.circles.shift();
    }
  },

  init() {
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    this.buffer.canvas.width = this.ctx.canvas.clientWidth;
    this.buffer.canvas.height = this.ctx.canvas.clientHeight;
    requestAnimationFrame(now => {
      this.lastTick = now;
      this.tick(now);
    });
    setInterval(() => this.generateCircle(), 100);
  }
};

gravity.init();