import Keyboard from "class/Keyboard";
import Spaceship from "class/Spaceship";
import Circle from "class/Circle";
import {getRandomInt} from "lib/Math";

const KEYBOARD = new Keyboard();

let miniGame = {
  ctx: $("canvas").get(0).getContext("2d"),
  lastTick: null,
  elapsedTime: 0,
  spaceship: null,
  targets: [],
  targets_maxInPlay: 10,
  target_radius: 20,
  gameLoss: false,
  moon: null,
  animationLoop: null,
  spawn_elapsedTime: 0,
  spawn_every: 1000,
  spawn_minDist: 200,

  tick(now) {
    this.animationLoop = requestAnimationFrame(now => this.tick(now));
    let deltaT = now - this.lastTick;
    this.lastTick = now;
    if (deltaT > 34) return; // if less than 30fps, drop the frame
    this.elapsedTime += deltaT;
    this.timer_spawn(deltaT);
    if (this.gameLoss) return;
    // Update world state
    this.processInput(deltaT);
    this.spaceship.move(deltaT, this.ctx);
    this.spaceship.applyFriction(deltaT);
    this.manageCollision();
    // Clear
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    // Redraw
    this.targets.forEach(t => t.draw(this.ctx));
    this.spaceship.draw(this.ctx);
    this.showScore();
  },

  timer_spawn(deltaT) {
    this.spawn_elapsedTime += deltaT;
    if (this.spawn_elapsedTime < this.spawn_every) return;
    this.spawn_elapsedTime = this.spawn_elapsedTime - this.spawn_every;
    this.spawnTarget();
  },

  spawnTarget() {
    let target = new Circle(
      getRandomInt(this.target_radius, this.ctx.canvas.width - this.target_radius),
      getRandomInt(this.target_radius, this.ctx.canvas.height - this.target_radius),
      this.target_radius
    );
    if (target.isCollidingWith(this.spaceship.x, this.spaceship.y, this.spawn_minDist-this.target_radius)) {
      this.spawnTarget();
      return;
    }
    this.targets.push(target);
    if (this.targets.length > this.targets_maxInPlay) {
      this.stop();
      this.loseTheGame();
    }
  },

  manageCollision() {
    this.targets = this.targets.filter(target => {
      return !target.isCollidingWith(this.spaceship.x, this.spaceship.y, this.spaceship.size/3);
    });
  },

  showScore() {
    let score = Math.round(this.elapsedTime / 100);
    this.ctx.font = "20px serif";
    this.ctx.fillStyle = "white";
    this.ctx.fillText(`Score: ${score}` , 60, 20);
  },

  loseTheGame() {
    this.gameLoss = true;
    this.ctx.font = "80px serif";
    this.ctx.textAlign = "center";
    this.ctx.textBaseline = "middle";
    this.ctx.fillStyle = "red";
    this.ctx.fillText("You lose !", this.ctx.canvas.width / 2, this.ctx.canvas.height / 2);
  },

  processInput(deltaT) {
    if (KEYBOARD.isKeyDown('d')) {
      this.spaceship.turn(deltaT, true);
    }
    if (KEYBOARD.isKeyDown('a')) {
      this.spaceship.turn(deltaT, false);
    }
    if (KEYBOARD.isKeyDown('w')) {
      this.spaceship.thrust(deltaT, false);
    }
    if (KEYBOARD.isKeyDown('s')) {
      this.spaceship.thrust(deltaT, true);
    }
    if (KEYBOARD.isKeyDown('space')) {
      console.log("fire!");
    }
  },

  init() {
    this.gameLoss = false;
    this.ctx.canvas.width = this.ctx.canvas.clientWidth;
    this.ctx.canvas.height = this.ctx.canvas.clientHeight;
    this.spaceship = new Spaceship(this.ctx.canvas.width / 2, this.ctx.canvas.height / 2, 30);
    this.start();
  },

  start() {
    this.animationLoop = requestAnimationFrame(now => {
      this.lastFrameTime = now;
      this.tick(now);
    });
  },

  stop() {
    cancelAnimationFrame(this.animationLoop);
    this.animationLoop = null;
  }

};

miniGame.init();