import Circle from "class/Circle/InFlatTorus";
import Keyboard from "class/Keyboard";
import {getRandomInt} from "lib/Math";

const KEYBOARD = new Keyboard(window);

const getAngleFromKeyPressed = () => {
  if (KEYBOARD.isKeysDown('a', 'w')) return Math.PI * 0.25;
  if (KEYBOARD.isKeysDown('a', 's')) return Math.PI * 1.75;
  if (KEYBOARD.isKeysDown('d', 'w')) return Math.PI * 0.75;
  if (KEYBOARD.isKeysDown('d', 's')) return Math.PI * 1.25;
  if (KEYBOARD.isKeyDown('s')) return Math.PI * 1.5;
  if (KEYBOARD.isKeyDown('d')) return Math.PI;
  if (KEYBOARD.isKeyDown('w')) return Math.PI * 0.5;
  if (KEYBOARD.isKeyDown('a')) return 0;
  return false;
}

const tick = (now, lastTime, circles, ctx) => {
  // request next frame
  requestAnimationFrame(time => tick(time, now, circles, ctx));
  let deltaT = now - lastTime;
  // update
  let angle = getAngleFromKeyPressed();
  if (angle !== false) {
    circles.forEach(c => c.move(angle, deltaT));
  }
  // clear (and resize)
  ctx.canvas.width = ctx.canvas.clientWidth;
  ctx.canvas.height = ctx.canvas.clientHeight;
  //draw
  circles.forEach(c => c.draw(ctx));
}

/*******************************************************************************
 * MAIN
 ******************************************************************************/
let ctx = $("canvas").get(0).getContext("2d");
ctx.canvas.width = ctx.canvas.clientWidth;
ctx.canvas.height = ctx.canvas.clientHeight;

let circles = [];
for (let i = 0; i < 200; i++) {
  let r = getRandomInt(3, Math.max(i / 4, 3));
  let c = new Circle(
    getRandomInt(0, ctx.canvas.width),
    getRandomInt(0, ctx.canvas.height),
    r,
    r / 100,
    ctx.canvas.width,
    ctx.canvas.height
    );
  circles.push(c);
}

circles.sort((c1, c2) => c1.compareToCircle(c2));

requestAnimationFrame(now => tick(now, now, circles, ctx));